/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "gui_fx.h"
#include "i18n.h"

#define COLOR(x) (x*(65535/255))

static GdkPixmap*	create_bg_pixmap 	(GtkWidget *widget, gdouble alpha);

void
gui_fx_set_blur_mode(GdkWindow* window)
{
	Display *xdisplay;
    long data[2];
    
    data[0] = 2; /* threshold */
    data[1] = 0; /* filter */
    
    xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());	
    
    XChangeProperty (xdisplay,
		     GDK_WINDOW_XID (window),
		     XInternAtom (xdisplay, "_COMPIZ_WM_WINDOW_BLUR", FALSE),
		     XA_INTEGER,
		     32, PropModeReplace, (guchar *) data,
		     2);	
}


static GdkPixmap*
create_bg_pixmap (GtkWidget *widget, gdouble alpha)
{	
	gint w, h;
    cairo_t	*cr;
    GdkPixmap *pixmap;
    
    gdk_drawable_get_size(widget->window, &w, &h);
    
    pixmap = gdk_pixmap_new(widget->window, w, h, -1);
    cr = gdk_cairo_create (pixmap);
    
    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);	
	cairo_set_source_rgba(cr, 1, 1, 1, alpha);	
  	cairo_paint(cr);
    cairo_destroy (cr);
    
    return pixmap;
}

void
gui_fx_set_alpha_mode(GtkWidget* wid)
{
    GdkColormap* colormap;
    
    colormap = gdk_screen_get_rgba_colormap (gdk_screen_get_default());
	
	if(colormap != NULL)
	    gtk_widget_set_colormap (wid, colormap);	
    else
        g_printerr(_("Failed to set colormap, no compositing window manager?\n"));
}

GtkStyle* 
gui_fx_transparency_style(GtkWidget *widget, gdouble alpha)
{
	gint i;
    GtkStyle *style;	    
    GdkPixmap *pixmap;        
    
    style = widget->style;
    
    if(!GTK_WIDGET_DRAWABLE(widget))
    	return style;
    
    pixmap = create_bg_pixmap (widget, alpha);        
    
    for (i = 0; i < 5; i++){
		style->bg_pixmap[i] = g_object_ref (pixmap);
    }

    return style; 
}


static GdkPixmap*
gui_fx_style_pixmap (GtkWidget *widget)
{       
    gint w, h;
    cairo_t     *cr;
    GdkPixmap *pixmap;
    
    gdk_drawable_get_size(widget->window, &w, &h);
    
    pixmap = gdk_pixmap_new (widget->window, w, h, -1);
    cr = gdk_cairo_create (pixmap);
    
    cairo_set_source_rgba(cr, 0.37, 0.37, 0.37, 1);     
    cairo_paint(cr);    
    cairo_destroy (cr);
    
    return pixmap;
}

GtkStyle* 
gui_fx_create_style(GtkWidget *widget)
{
    gint i;
    GtkStyle *style;        
    GdkPixmap *pixmap;        
    
    if(GTK_WIDGET_DRAWABLE(widget))
        pixmap = gui_fx_style_pixmap (widget);        
    else
        pixmap = NULL;
    
    style = widget->style;
    
    for (i = 0; i < 5; i++){            
        if(pixmap)
            style->bg_pixmap[i] = g_object_ref (pixmap);
        
        style->fg[i].red = COLOR(255);
        style->fg[i].blue = COLOR(255);
        style->fg[i].green = COLOR(255);
        style->fg[i].pixel = (gulong)COLOR(255)+COLOR(255)+COLOR(255);    
        
        style->text[i].red = COLOR(41);
        style->text[i].blue = COLOR(41);
        style->text[i].green = COLOR(41);
        style->text[i].pixel = (gulong)COLOR(41)+COLOR(41)+COLOR(41);    
        
        style->bg[i].red = COLOR(141);
        style->bg[i].blue = COLOR(141);
        style->bg[i].green = COLOR(141);
        style->bg[i].pixel = (gulong)COLOR(141)+COLOR(141)+COLOR(141);
    }
    
    return style;
}

void 
gui_fx_set_and_create_style(GtkWidget *widget)
{
    if(widget == NULL) return;
    
    GtkStyle *style = gui_fx_create_style(widget);
    
    if (GTK_IS_BOX(widget)){
        GtkBoxChild* child;
        GList *children = g_list_first(GTK_BOX(widget)->children);
        for(;children; children = children->next){
            if(children->data){
                child = children->data;
                if(child->widget != NULL)
                    gui_fx_set_and_create_style(child->widget);
            }
        }
    }   
    
    if(GTK_IS_BIN (widget)){            
        GtkWidget *child = GTK_BIN (widget)->child;
        if(child != NULL)
            gui_fx_set_and_create_style (child);
    }
    
    gtk_widget_set_style (widget, style);    
}
