/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "io_ipc.h"
#include "io_settings.h"
#include "gui_middle_area.h"
#include "gui_middle_area_marshallers.h"

#define LINE_SPACE 10
#define LINE_OFFSET_DOT 12
#define LINE_OFFSET_Y_CLI 14
#define LINE_OFFSET_Y_NET 15
#define LINE_OFFSET_X_CLI 33
#define LINE_OFFSET_X_NET 18
#define TEXT_SIZE_NET 12
#define TEXT_SIZE_CLI 10

#define STD_HIGHT 260

#define DOT_ONLINE "/dot_online.png"
#define DOT_OFFLINE "/dot_offline.png"

#define GLOBAL_ALPHA 1

#define GUI_MIDDLE_AREA_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GUI_TYPE_MIDDLE_AREA, GuiMiddleAreaPrivate))

G_DEFINE_TYPE (GuiMiddleArea, gui_middle_area, GTK_TYPE_DRAWING_AREA);

typedef struct Row{
	IPCNetwork *network;
	IPCClient *client; 
	
	gint ystart;
	gint ystop;
	
	struct Row *next;	
}Row;

static gboolean gui_middle_area_expose 			(GtkWidget *middle_area, GdkEventExpose *event);
static gboolean gui_middle_area_configure_event	(GtkWidget *middle_area, GdkEventConfigure *event);
static gboolean gui_middle_area_button_release 	(GtkWidget *middle_area, GdkEventButton *event);
static gboolean gui_middle_area_button_press	(GtkWidget *middle_area, GdkEventButton *event);
static void 	gui_middle_area_grab_notify     (GtkWidget *middle_area, gboolean was_grabbed);

static void gui_middle_area_finalize 			(GuiMiddleArea *middle_area);
static void	gui_middle_area_settings_updated	(IOSettings *settings, GuiMiddleArea *middle_area);
static void	gui_middle_area_draw_dot			(cairo_t *cr, gint x, gint y, gboolean online);
static void gui_middle_area_draw_base			(GtkWidget *widget, cairo_t *cr);
static void gui_middle_area_draw_network		(GtkWidget *widget, cairo_t *cr);
static void gui_middle_area_draw_active_row		(GtkWidget *widget, cairo_t *cr);
static void gui_middle_area_size_request  		(GtkWidget *widget, GtkRequisition *requisition);
static void gui_middle_area_calc_addr_width     (GtkWidget* widget, cairo_t *cr);
static void gui_middle_area_parse_and_set_font  (GtkWidget* widget, cairo_t *cr, cairo_font_slant_t slant, cairo_font_weight_t weight);

static Row*	gui_middle_area_draw_network_line	(GtkWidget* widget, IPCNetwork *net, cairo_t *cr, gint y);
static Row* gui_middle_area_draw_client_line	(GtkWidget* widget, IPCClient *client, cairo_t *cr, gint y);

typedef struct _GuiMiddleAreaPrivate GuiMiddleAreaPrivate;

struct _GuiMiddleAreaPrivate
{
	IPCNetwork *list;
	Row *rows;
	GdkColor *bg_color;
	GdkRectangle *active_row;
	
	gint width;
	gint height;
	gint ip_addr_width;
	gchar* fontstr;
	gdouble alpha;	
	gboolean resized;
	gboolean show_offline;
	IOSettings *settings;
};

enum{
	SIGNAL_NETWORK_ROW_ACTIVATED,
	SIGNAL_CLIENT_ROW_ACTIVATED,
	SIGNAL_LAST
};

static guint gui_middle_area_signals[SIGNAL_LAST] = { 0 , 0 };

static void
gui_middle_area_class_init (GuiMiddleAreaClass *class)
{
	GObjectClass *obj_class;
	GtkWidgetClass *widget_class;

	obj_class = G_OBJECT_CLASS (class);
	obj_class->finalize = (GObjectFinalizeFunc) gui_middle_area_finalize;	

	widget_class = GTK_WIDGET_CLASS (class);
	widget_class->expose_event = gui_middle_area_expose;
	widget_class->configure_event = gui_middle_area_configure_event;
	widget_class->button_release_event = gui_middle_area_button_release;
	widget_class->button_press_event = gui_middle_area_button_press;
  	widget_class->grab_notify = gui_middle_area_grab_notify;
  	widget_class->size_request = gui_middle_area_size_request;
  	
	gui_middle_area_signals[SIGNAL_NETWORK_ROW_ACTIVATED] = g_signal_new (
					"network-row-activated",
					G_OBJECT_CLASS_TYPE (obj_class),
					G_SIGNAL_RUN_FIRST,
					0, NULL, NULL,
					_middle_area_marshal_VOID__POINTER_UINT,
					G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_UINT);

	gui_middle_area_signals[SIGNAL_CLIENT_ROW_ACTIVATED] = g_signal_new (
					"client-row-activated",
					G_OBJECT_CLASS_TYPE (obj_class),
					G_SIGNAL_RUN_FIRST,
					0, NULL, NULL,
					_middle_area_marshal_VOID__POINTER_UINT,
					G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_UINT);

	g_type_class_add_private (obj_class, sizeof (GuiMiddleAreaPrivate));
}

static void
gui_middle_area_init (GuiMiddleArea *middle_area)
{
	GuiMiddleAreaPrivate *priv;

	GTK_WIDGET_SET_FLAGS (middle_area, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT | GTK_DOUBLE_BUFFERED);

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (middle_area);
	priv->list = NULL;
	priv->rows = NULL;
	priv->active_row = NULL;
	priv->width = 200;
	priv->height = STD_HIGHT;	
	priv->resized = FALSE;
	priv->alpha = 1;
	
	gtk_widget_set_extension_events (GTK_WIDGET (middle_area), GDK_EXTENSION_EVENTS_CURSOR);

	gtk_widget_add_events (GTK_WIDGET (middle_area), GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_BUTTON_MOTION_MASK
			 | GDK_POINTER_MOTION_MASK);
}

static void
gui_middle_area_finalize (GuiMiddleArea *top_area)
{
	/** TODO: fix finalize() */
}

static gboolean
gui_middle_area_configure_event (GtkWidget *middle_area, GdkEventConfigure *event)
{
	return FALSE;
}

static void
gui_middle_area_size_request (GtkWidget *widget, GtkRequisition *requisition)
{
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (widget);
	
	requisition->width = priv->width;
	requisition->height = priv->height;	
}

static gboolean
gui_middle_area_expose (GtkWidget *widget, GdkEventExpose *event)
{
	cairo_t *cr;
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (widget);    
    
    cr = gdk_cairo_create(widget->window);	
       
    cairo_rectangle(cr, event->area.x, event->area.y,
		    		event->area.width, event->area.height);
    cairo_clip(cr);	
    
	gui_middle_area_draw_base(widget, cr);	
	
	cairo_rectangle(cr, 2, 2, 
					widget->allocation.width - 4, widget->allocation.height - 4);
    cairo_clip(cr);	

	gui_middle_area_draw_active_row(widget, cr);		
	gui_middle_area_draw_network(widget, cr);
	
	cairo_destroy(cr);

	return TRUE;
}

static gboolean
gui_middle_area_button_press(GtkWidget *middle_area, GdkEventButton *event)
{
	Row *row;
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (GUI_MIDDLE_AREA(middle_area));	
	
	if(!priv->rows) return FALSE;
	
	if(priv->active_row){ 
		g_free(priv->active_row);
		priv->active_row = NULL;
	}	
	
	for(row = priv->rows; row; row = row->next){
		if((event->y > row->ystart) &&
		   (event->y < row->ystop)){
			if(row->network){				
				g_signal_emit (GUI_MIDDLE_AREA(middle_area),
							gui_middle_area_signals[SIGNAL_NETWORK_ROW_ACTIVATED], 0,
							row->network, event->button);
			}else if(row->client){				
				g_signal_emit (GUI_MIDDLE_AREA(middle_area),
							gui_middle_area_signals[SIGNAL_CLIENT_ROW_ACTIVATED], 0,
							row->client, event->button);
			}			
			
			priv->active_row = g_malloc0(sizeof(GdkRectangle));
			priv->active_row->x = 0;
			priv->active_row->y = row->ystart - 4;
			priv->active_row->width = middle_area->allocation.width;
			priv->active_row->height = row->ystop - row->ystart + 8;
		
			gdk_window_invalidate_rect(middle_area->window, priv->active_row, FALSE);
		}
	}	
	
	return TRUE;
}

static gboolean
gui_middle_area_button_release (GtkWidget *middle_area, GdkEventButton *event)
{
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (GUI_MIDDLE_AREA(middle_area));
	
	if(priv->active_row){							
		gdk_window_invalidate_rect(middle_area->window, priv->active_row, FALSE);
		g_free(priv->active_row);
		priv->active_row = NULL;		
	}

	return TRUE;
}

static void
gui_middle_area_grab_notify(GtkWidget *middle_area, gboolean was_grabbed)
{
	GuiMiddleAreaPrivate *priv;
	
	priv = GUI_MIDDLE_AREA_GET_PRIVATE (GUI_MIDDLE_AREA(middle_area));
	
	if(priv->active_row){							
		gdk_window_invalidate_rect(middle_area->window, priv->active_row, FALSE);
		g_free(priv->active_row);
		priv->active_row = NULL;		
	}
}

static void
gui_middle_area_draw_base(GtkWidget *widget, cairo_t *cr)
{
    GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (GUI_MIDDLE_AREA(widget));	
    
	cairo_save(cr);
	cairo_set_line_width(cr, 1);	
	
	/**	*/
	gdk_cairo_set_source_color(cr, priv->bg_color);
	cairo_paint_with_alpha(cr, priv->alpha);
 	
     /** left frame extern SW -> NW -> NE */
    cairo_set_source_rgba(cr, 1, 1, 1, priv->alpha);	  	
  	cairo_move_to(cr, 0.5, 0);
  	cairo_line_to(cr, 0.5, widget->allocation.height);
 	cairo_stroke(cr);
 	
 	/** left frame SW -> NW -> NE */
 	cairo_set_source_rgba(cr, 0.56, 0.56, 0.56, priv->alpha);	
	cairo_move_to(cr, 1.5, widget->allocation.height);
  	cairo_line_to(cr, 1.5, 0.5);
  	cairo_line_to(cr, widget->allocation.width-2, 0.5); 	
 	cairo_stroke(cr);
  	
  	/** left black line NE -> SE */
  	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, priv->alpha);	
  	cairo_move_to(cr, widget->allocation.width-0.5, 0);
  	cairo_line_to(cr, widget->allocation.width-0.5, widget->allocation.height);
  	cairo_stroke(cr);
	
	/** right frame NE -> SE -> SW */
	cairo_set_source_rgba(cr, 0.184, 0.184, 0.184, priv->alpha);
	cairo_move_to(cr, widget->allocation.width-1.5, 0);
	cairo_line_to(cr, widget->allocation.width-1.5, widget->allocation.height-0.5);	
	cairo_line_to(cr, 1, widget->allocation.height-0.5);
	cairo_stroke(cr);
  	
	cairo_restore(cr);
}

static void
gui_middle_area_draw_dot(cairo_t *cr, gint x, gint y, gboolean online)
{
	gchar *file;
	GdkPixbuf* pixb;
	
	if(online)
  		file = PACKAGE_DATA_DIR DOT_ONLINE;
	else
  		file = PACKAGE_DATA_DIR DOT_OFFLINE;
	
	pixb = gdk_pixbuf_new_from_file (file, NULL);

	g_return_if_fail(pixb);
	
	cairo_save(cr);
	cairo_set_operator (cr, CAIRO_OPERATOR_ATOP);	
	gdk_cairo_set_source_pixbuf(cr, pixb, x, y - 3);	
	cairo_paint_with_alpha(cr, GLOBAL_ALPHA);
	cairo_restore(cr);  
	g_object_unref (pixb);	
}

static void
gui_middle_area_draw_active_row(GtkWidget *widget, cairo_t *cr)
{
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (GUI_MIDDLE_AREA(widget));
	
	if(!priv->active_row) return;
	
	cairo_save(cr);
	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
	cairo_set_source_rgba(cr, 0.39, 0.40, 0.39, priv->alpha);	
	gdk_cairo_rectangle(cr, priv->active_row);
	cairo_fill(cr);
	cairo_restore(cr);
}

static void
gui_middle_area_parse_and_set_font(GtkWidget* widget, cairo_t *cr, cairo_font_slant_t slant, cairo_font_weight_t weight)
{
    gint i = 0;
    gchar** matches;
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (GUI_MIDDLE_AREA(widget));        
    
	if(priv->fontstr == NULL){
	    cairo_select_font_face(cr, "Georgia", slant, weight);
	    if(weight == CAIRO_FONT_WEIGHT_NORMAL)
            cairo_set_font_size(cr, TEXT_SIZE_CLI);	
        else
            cairo_set_font_size(cr, TEXT_SIZE_NET);	
	    return;
    }

    matches = g_regex_split_simple("(?P<fam>.+? )(?P<bold>Bold |Bold Italic |Italic |)(?P<fontsize>\\d+)", priv->fontstr, 0, 0);        
    
    if(matches[1] != NULL)
        cairo_select_font_face(cr, matches[1], slant, weight);
    else
        cairo_select_font_face(cr, "Georgia", slant, weight);

    while(TRUE){
        if(matches[i + 1] == NULL) break;
        i++;
    }
    
    if(matches[i - 1] == NULL){
        if(weight == CAIRO_FONT_WEIGHT_NORMAL)
            cairo_set_font_size(cr, TEXT_SIZE_CLI);	
        else
            cairo_set_font_size(cr, TEXT_SIZE_NET);	
    } else {
        if(weight == CAIRO_FONT_WEIGHT_NORMAL)
            cairo_set_font_size(cr, atoi(matches[i - 1]));	
        else
            cairo_set_font_size(cr, atoi(matches[i - 1]) + 2);	
    }

    g_strfreev(matches);
}

static void
gui_middle_area_calc_addr_width(GtkWidget* widget, cairo_t *cr)
{
    IPCNetwork *net;
	IPCClient *client;
	cairo_text_extents_t te;
    GuiMiddleAreaPrivate *priv;    

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (widget);
	priv->ip_addr_width = 0;
	
	cairo_save(cr);
	gui_middle_area_parse_and_set_font(widget, cr, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
	
    for(net = priv->list; net; net = net->next){				
		for(client = net->clients; client; client = client->next){	
		    cairo_text_extents(cr, client->addr, &te);
		    if(priv->ip_addr_width < te.width) priv->ip_addr_width = te.width;
		}
	}
	
	cairo_restore(cr);
}

static Row*
gui_middle_area_draw_client_line(GtkWidget *widget, IPCClient *client, cairo_t *cr, gint y)
{
	Row* row;
	cairo_text_extents_t te;
	GuiMiddleAreaPrivate *priv;    

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (widget);
	
	cairo_save(cr);		
	
	gui_middle_area_draw_dot(cr, LINE_OFFSET_X_CLI - LINE_OFFSET_DOT, y, client->online);
	
	cairo_set_source_rgb (cr, 1, 1, 1);
	gui_middle_area_parse_and_set_font(widget, cr, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
	
	cairo_text_extents(cr, client->nick, &te);
	cairo_move_to(cr, LINE_OFFSET_X_CLI, y + (te.height / 2));
	if(client->addr) cairo_show_text(cr, client->addr);	
	cairo_move_to(cr, LINE_OFFSET_X_CLI*1.5 + priv->ip_addr_width, y + (te.height / 2));
	if(client->nick) cairo_show_text(cr, client->nick);	
	
	if(priv->width < (LINE_OFFSET_X_CLI*1.5 + priv->ip_addr_width + te.width))
	    priv->width = (LINE_OFFSET_X_CLI*1.5 + priv->ip_addr_width + te.width);
	
	cairo_restore(cr);
		
	row = g_malloc0(sizeof(Row));
	row->network = NULL;
	row->client = client;
	row->ystart =  y - (te.height / 2);
	row->ystop = y + (te.height / 2);
	
	return row;
}

static Row*
gui_middle_area_draw_network_line(GtkWidget *widget, IPCNetwork *net, cairo_t *cr, gint y)
{
	Row* row;
	cairo_text_extents_t te;
	
	cairo_save(cr);		
	
	gui_middle_area_draw_dot(cr, LINE_OFFSET_X_NET - LINE_OFFSET_DOT, y, net->online);
	
	cairo_set_source_rgb (cr, 1, 1, 1);
	gui_middle_area_parse_and_set_font(widget, cr, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
	
	cairo_text_extents(cr, net->name, &te);
	cairo_move_to(cr, LINE_OFFSET_X_NET, y + (te.height / 2));
	cairo_show_text(cr, net->name);	
	
	cairo_restore(cr);
		
	row = g_malloc0(sizeof(Row));
	row->network = net;
	row->client = NULL;
	row->ystart =  y - (te.height / 2);
	row->ystop = y + (te.height / 2);
	
	return row;
}

static void
gui_middle_area_draw_network(GtkWidget *widget, cairo_t *cr)
{
	gint cy = 20;
	Row *row = NULL;
	IPCNetwork *net;
	IPCClient *client;
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (widget);
	priv->width = 0; /** will be recalced in gui_middle_area_draw_client_line */
	
	if(priv->rows){
		for(;priv->rows; priv->rows = priv->rows->next){
			if(row) g_free(row);			
			row = priv->rows;
		}
		if(row) g_free(row);
		priv->rows = row = NULL;
	}
	
	gui_middle_area_calc_addr_width(widget, cr);
	
	for(net = priv->list; net; net = net->next){				
		if(row){
			row->next = gui_middle_area_draw_network_line(widget, net, cr, cy);
			row = row->next;
			cy = LINE_OFFSET_Y_NET + row->ystop; 
		}else{
			row = gui_middle_area_draw_network_line(widget, net, cr, cy);
			cy = LINE_OFFSET_Y_NET + row->ystop; 
		}
		if(!priv->rows)
			priv->rows = row;
				
		for(client = net->clients; client; client = client->next){	
			if(client->online || !priv->show_offline){		
				row->next = gui_middle_area_draw_client_line(widget, client, cr, cy);
				row = row->next;
			}
			
			cy = LINE_OFFSET_Y_CLI + row->ystop; 
		}
	}
	
	if((priv->height != cy) && (cy > STD_HIGHT)){
		priv->height = cy;
		gtk_widget_queue_resize(widget);
	}
}

static void
gui_middle_area_settings_updated(IOSettings *settings, GuiMiddleArea *middle_area)
{
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (middle_area);	
	priv->bg_color = &settings->main_window_color;
	priv->show_offline = settings->show_offline;
	priv->fontstr = settings->fontstr;

	if(settings->show_opacity)
		priv->alpha = settings->opacity;
	else
		priv->alpha = 1;        
    
	gtk_widget_queue_draw(GTK_WIDGET(middle_area));
}

void
gui_middle_area_set_network(GuiMiddleArea *middle_area, IPCNetwork* list)
{
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (middle_area);
	priv->list = list;
	
	gtk_widget_queue_draw(GTK_WIDGET(middle_area));
}

void
gui_middle_area_set_settings(GuiMiddleArea *middle_area, IOSettings *settings)
{
	GuiMiddleAreaPrivate *priv;

	priv = GUI_MIDDLE_AREA_GET_PRIVATE (middle_area);
	priv->settings = settings;		
	priv->show_offline = settings->show_offline;
	
	g_return_if_fail(settings);	
	g_signal_connect(settings, "settings-update",
					 G_CALLBACK(gui_middle_area_settings_updated), middle_area);					 					 
}

GtkWidget*
gui_middle_area_new (IOSettings *settings)
{
	GuiMiddleArea *middle_area;
	
	middle_area = g_object_new (GUI_TYPE_MIDDLE_AREA, NULL);
	
	gui_middle_area_set_settings(middle_area, settings);
	
	return GTK_WIDGET(middle_area);
}

