/*
 *  Copyright (C) 2008 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glade/glade-xml.h>
#include "gui_settings_window.h"
#include "gui_settings_window_callbacks.h"
#include "gui_text_scrolling.h"
#include "gui_frame.h"
#include "gui_fx.h"
#include "io_settings.h"
#include "io_misc.h"
#include "config.h"
#include "main.h"
#include "i18n.h"

#define PIXMAP_SYSTEM "/system.png"
#define PIXMAP_WINDOW "/window.png"
#define PIXMAP_STATUS "/status.png"
#define PIXMAP_ABOUT "/about.png"

#define GLADE_SYSTEM "/system.glade"
#define GLADE_WINDOW "/window.glade"
#define GLADE_STATUS "/status.glade"
#define GLADE_WIDGET "window-box"

enum {
    COLUMN_PIXBUF,
    COLUMN_STRING,
    COLUMN_TYPE,
    N_COLUMNS
};

enum {
    SYSTEM,
    WINDOW,
    STATUS,
    ABOUT
};

static GtkWidget* top_box;
static GtkWidget* main_window;
static GtkWidget* side_system;
static GtkWidget* side_window;
static GtkWidget* side_status;
static GtkWidget* side_current;
static GtkWidget* side_about;

static GtkWidget*
gui_settings_load_window(IOSettings *settings)
{
	GladeXML* xml;    
    GtkWidget *wid;
    GtkWidget *box;
    
    xml = glade_xml_new(PACKAGE_RESOURCES_DIR GLADE_WINDOW, GLADE_WIDGET, NULL);
    box = glade_xml_get_widget(xml, GLADE_WIDGET);
    
    wid = glade_xml_get_widget(xml, "trans-effect");    
    g_signal_connect(wid, "toggled", G_CALLBACK(gui_settings_window_effect_callback), settings);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), settings->show_opacity);
    
    wid = glade_xml_get_widget(xml, "transparency-amount");    
    g_signal_connect(wid, "value-changed", G_CALLBACK(gui_settings_window_opacity_callback), settings);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(wid), settings->opacity);
    
    wid = glade_xml_get_widget(xml, "show-taskbar");    
    g_signal_connect(wid, "toggled", G_CALLBACK(gui_settings_window_show_taskbar_callback), settings);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), settings->show_taskbar);
    
    wid = glade_xml_get_widget(xml, "show-offline");    
    g_signal_connect(wid, "toggled", G_CALLBACK(gui_settings_window_show_offline_callback), settings);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), settings->show_offline);
    
    wid = glade_xml_get_widget(xml, "main-window-colorbutton");    
    g_signal_connect(wid, "color-set", G_CALLBACK(gui_settings_window_color_callback), settings);
    gtk_color_button_set_color(GTK_COLOR_BUTTON(wid), &settings->main_window_color);

    wid = glade_xml_get_widget(xml, "main-window-fontbutton");    
    g_signal_connect(wid, "font-set", G_CALLBACK(gui_settings_window_font_callback), settings);    
    gtk_font_button_set_font_name(GTK_FONT_BUTTON(wid), settings->fontstr);
    
    wid = glade_xml_get_widget(xml, "show-quit-dialog");    
    g_signal_connect(wid, "toggled", G_CALLBACK(gui_settings_window_quit_dialog_callback), settings);    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), settings->show_quit_dialog);
    
    gui_fx_set_and_create_style(box);
    return box;
}
 
static GtkWidget*
gui_settings_load_status(MainPrivate* priv)
{
	GladeXML* xml;
    GtkWidget *box;
    GtkWidget *wid;
    
    xml = glade_xml_new(PACKAGE_RESOURCES_DIR GLADE_STATUS, GLADE_WIDGET, NULL);
    box = glade_xml_get_widget(xml, GLADE_WIDGET);    
    
    wid = glade_xml_get_widget(xml, "address");    
    gtk_label_set_label(GTK_LABEL(wid), io_misc_get_addr());
    
    wid = glade_xml_get_widget(xml, "gui");
    gtk_label_set_label(GTK_LABEL(wid), VERSION);
    
    wid = glade_xml_get_widget(xml, "date");
    gtk_label_set_label(GTK_LABEL(wid), __DATE__);
    
    wid = glade_xml_get_widget(xml, "version");
    gtk_label_set_label(GTK_LABEL(wid), priv->stats->useragent + 12);
    
    wid = glade_xml_get_widget(xml, "nickname");
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_nick_callback), priv);
    gtk_entry_set_text(GTK_ENTRY(wid), priv->stats->nick);
    
    gui_fx_set_and_create_style(wid);    
    gui_fx_set_and_create_style(box);
    return box;
}

static GtkWidget*
gui_settings_load_system(MainPrivate* priv)
{
    gchar buff[1024];
	GladeXML* xml;
    GtkWidget *box;
    GtkWidget *wid;
    
    xml = glade_xml_new(PACKAGE_RESOURCES_DIR GLADE_SYSTEM, GLADE_WIDGET, NULL);
    box = glade_xml_get_widget(xml, GLADE_WIDGET);     
    
    wid = glade_xml_get_widget(xml, "vnc");    
    gtk_entry_set_text(GTK_ENTRY(wid), priv->settings->cmd_vnc);
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_vnc_entry_callback), priv->settings);        
    
    wid = glade_xml_get_widget(xml, "sudosu");    
    gtk_entry_set_text(GTK_ENTRY(wid), priv->settings->cmd_sudo);
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_sudo_entry_callback), priv->settings);            

    wid = glade_xml_get_widget(xml, "browser");    
    gtk_entry_set_text(GTK_ENTRY(wid), priv->settings->cmd_browser);
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_browser_entry_callback), priv->settings); 
    
    wid = glade_xml_get_widget(xml, "hamachi");    
    gtk_entry_set_text(GTK_ENTRY(wid), priv->settings->cmd_hamachi);
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_hamachi_entry_callback), priv->settings);
    
    wid = glade_xml_get_widget(xml, "tuncfg");    
    gtk_entry_set_text(GTK_ENTRY(wid), priv->settings->cmd_tuncfg);
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_tuncfg_entry_callback), priv->settings);
    
    wid = glade_xml_get_widget(xml, "terminal");    
    gtk_entry_set_text(GTK_ENTRY(wid), priv->settings->cmd_terminal);
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_terminal_entry_callback), priv->settings);   
    
    wid = glade_xml_get_widget(xml, "hamachifolder");    
    gtk_entry_set_text(GTK_ENTRY(wid), priv->settings->hamachi_folder);
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_hamachi_folder_entry_callback), priv->settings);
    
    wid = glade_xml_get_widget(xml, "keepalive");    
    if(priv->hsettings->keep_alive == -1){
        gtk_entry_set_text(GTK_ENTRY(wid), "");
    }else{
        sprintf((gchar*)&buff, "%d", priv->hsettings->keep_alive);
        gtk_entry_set_text(GTK_ENTRY(wid), (gchar*)&buff);
    }
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_keepalive_entry_callback), priv);

    wid = glade_xml_get_widget(xml, "udpport");    
    if(priv->hsettings->udp_port == -1){
        gtk_entry_set_text(GTK_ENTRY(wid), "");
    }else{
        sprintf((gchar*)&buff, "%d", priv->hsettings->udp_port);
        gtk_entry_set_text(GTK_ENTRY(wid), (gchar*)&buff);
    }
    g_signal_connect(wid, "focus-out-event", G_CALLBACK(gui_settings_window_udpport_entry_callback), priv);
    
    gui_fx_set_and_create_style(box);
    return box;
}

static void
gui_settings_add_person(GtkWidget* wid, gchar* name, gchar* what)
{
    gui_text_scrolling_set_text(wid, "");
    gui_text_scrolling_set_text(wid, name);
    gui_text_scrolling_set_text(wid, what);
    
    g_free(what);
}

static GtkWidget*
gui_settings_load_about(IOSettings *settings)
{
    gchar* ideas = _("Ideas");
    gchar* testing = _("Testing");
    gchar* feedback = _("Feedback");
    gchar* translations = _("Translations");
    GtkWidget* wid;
    
    wid = gui_text_scrolling_new();
    gui_text_scrolling_set_text(wid, PACKAGE);
    gui_text_scrolling_set_text(wid, VERSION);
    gui_text_scrolling_set_text(wid, "(" __DATE__ ")");
    gui_text_scrolling_set_text(wid, "");
    gui_text_scrolling_set_text(wid, _("Creator"));
    gui_text_scrolling_set_text(wid, _("Developer"));
    gui_text_scrolling_set_text(wid, _("Project Maintainer:"));
    gui_text_scrolling_set_text(wid, "");
    gui_text_scrolling_set_text(wid, "Viktor Nordell");
    gui_text_scrolling_set_text(wid, "");
    gui_text_scrolling_set_text(wid, _("Thanks to:"));
        
    gui_settings_add_person(wid, "giopas", g_strconcat("(", testing, ", ", ideas, ", ", translations,")", NULL));
    gui_settings_add_person(wid, "Jonas Sandell", g_strconcat("(", testing, ", ", ideas, ")", NULL));
    gui_settings_add_person(wid, "paul-lmi", g_strconcat("(", feedback, ", ", ideas, ")", NULL));
    gui_settings_add_person(wid, "Marco Di Antonio", g_strconcat("(", testing, ", ", ideas, ")", NULL));
    gui_settings_add_person(wid, "Jan Rembisch", g_strconcat("(", testing, ", ", translations, ")", NULL));
    gui_settings_add_person(wid, "Steve Reinhold", g_strconcat("(", translations, ")", NULL));    
    gui_settings_add_person(wid, "Barbro Storm", g_strconcat("(", translations, ")", NULL));    
    gui_settings_add_person(wid, "Rodrigo Linfati", g_strconcat("(", translations, ")", NULL));  
    gui_settings_add_person(wid, "Łukasz Taczuk", g_strconcat("(", translations, ")", NULL));      
    gui_settings_add_person(wid, "Svetoslav Stefanov", g_strconcat("(", translations, ")", NULL));  
    gui_settings_add_person(wid, "Jozef Riha", g_strconcat("(", translations, ")", NULL));      
    gui_settings_add_person(wid, "Max Walter", g_strconcat("(", testing, ")", NULL));    
    
    gui_text_scrolling_set_text(wid, "");
    gui_text_scrolling_set_text(wid, _("And of course"));
    gui_text_scrolling_set_text(wid, "Alex Pankratov");
    gui_text_scrolling_set_text(wid, _("(hamachi, protocol definition)"));
    gtk_widget_show(wid);
    
    return wid;
}

static void
gui_setting_load_list_item(GtkListStore *list, GtkTreeIter *iter, gchar* pix, gchar* str, gint type)
{
    gchar* path;
    GdkPixbuf* img;
    
    path = g_strconcat(PACKAGE_DATA_DIR, pix, NULL);    
    g_return_if_fail(path);
    
    img = gdk_pixbuf_new_from_file (path, NULL);
    g_return_if_fail(img);
    
    gtk_list_store_append (list, iter);
    gtk_list_store_set (list, iter, COLUMN_PIXBUF, img, COLUMN_STRING, str, COLUMN_TYPE, type, -1);
    
    g_free(path);                                               
    /** TODO free mem */
}

static GtkListStore* 
gui_settings_create_list()
{
    GtkTreeIter iter;
    GtkListStore *list; 
    
    list = gtk_list_store_new (N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_INT);
    
    gui_setting_load_list_item(list, &iter, PIXMAP_STATUS, _("Status"), STATUS);
    gui_setting_load_list_item(list, &iter, PIXMAP_WINDOW, _("Window"), WINDOW);
    gui_setting_load_list_item(list, &iter, PIXMAP_SYSTEM, _("System"), SYSTEM);
    gui_setting_load_list_item(list, &iter, PIXMAP_ABOUT, _("About"), ABOUT);
    
    return list;                                                
}

static void
gui_settings_window_item_activated (GtkIconView *icon_view, gpointer user_data)
{
    GList* selected;
    GtkListStore *store;
    GtkTreeIter iter;
    GtkWidget* side;
    gint type;
    
    store = GTK_LIST_STORE(user_data);
    selected = gtk_icon_view_get_selected_items(icon_view);
    
    if(!selected || !gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, selected->data)){
        g_list_foreach (selected, (GFunc)gtk_tree_path_free, NULL);
        g_list_free (selected);
        return;
    }
    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COLUMN_TYPE, &type, -1);    
        
    switch(type){
    case STATUS:                
        side = side_status;             
        gui_text_scrolling_stop(side_about);
        break;
    case WINDOW:
        side = side_window;      
        gui_text_scrolling_stop(side_about);
        break;
    case SYSTEM:
        side = side_system;
        gui_text_scrolling_stop(side_about);
        break;
    case ABOUT:
        side = side_about;
        gui_text_scrolling_start(side_about);
        break;
    default:
        return;
    }
    
    if(side_current != side){        
        gtk_container_remove(GTK_CONTAINER(top_box), side_current);
        gtk_container_add(GTK_CONTAINER(top_box), side);
        side_current = side;
    }
    
    g_list_foreach (selected, (GFunc)gtk_tree_path_free, NULL);
    g_list_free (selected);
    gtk_icon_view_unselect_all(icon_view);
}

static void
gui_frame_right_button_clicked(GuiFrame *frame)
{	
	gtk_widget_hide_all(main_window);
	gui_text_scrolling_stop(side_about);
}

static void
gui_settings_window_create(MainPrivate* priv)
{
    GtkWidget *view;
    GtkWidget *frame;           
    GtkListStore *list; 
    
    list = gui_settings_create_list();
    view = gtk_icon_view_new_with_model(GTK_TREE_MODEL(list));
    frame = gui_frame_new();
    
    top_box = gtk_hbox_new(FALSE, 0);
    main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);  
    
    side_window = gui_settings_load_window(priv->settings);    
    side_system = gui_settings_load_system(priv);
    side_status = gui_settings_load_status(priv);   
    side_about = gui_settings_load_about(priv->settings);
    side_current = side_status;
    g_object_ref(side_window);
    g_object_ref(side_status);
    g_object_ref(side_system);
    g_object_ref(side_about);
    
    gtk_icon_view_set_markup_column(GTK_ICON_VIEW(view), COLUMN_STRING);
    gtk_icon_view_set_pixbuf_column(GTK_ICON_VIEW(view), COLUMN_PIXBUF);
    gtk_icon_view_set_selection_mode(GTK_ICON_VIEW(view), GTK_SELECTION_SINGLE);
    
    g_signal_connect (view, "selection-changed", G_CALLBACK (gui_settings_window_item_activated), list);
    
    gui_fx_set_and_create_style(view);
    
    gtk_box_pack_start(GTK_BOX(top_box), view, FALSE, FALSE, 10);
    gtk_container_add(GTK_CONTAINER(top_box), side_status);     
    
    gtk_container_add(GTK_CONTAINER(frame), top_box);
    gtk_container_add(GTK_CONTAINER(main_window), frame);
    
    g_signal_connect (frame, "right-button-clicked",
        G_CALLBACK (gui_frame_right_button_clicked), NULL);	
    
    gtk_window_set_decorated(GTK_WINDOW(main_window), FALSE);
    gtk_widget_show_all(main_window);
}

void
gui_settings_window_show(MainPrivate* priv)
{
    if(main_window == NULL){
        gui_settings_window_create(priv);
    }else{
        gtk_widget_show_all(GTK_WIDGET(main_window));
        gui_text_scrolling_start(side_about);
    }   
}

