/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "gui_text_scrolling.h"

#define GUI_TEXT_SCROLLING_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GUI_TYPE_TEXT_SCROLLING, GuiTextScrollingPrivate))

G_DEFINE_TYPE (GuiTextScrolling, gui_text_scrolling, GTK_TYPE_DRAWING_AREA);

static void  gui_text_scrolling_finalize (GuiTextScrolling *text_scrolling);

static gboolean gui_text_scrolling_expose 			(GtkWidget *text_scrolling, GdkEventExpose *event);
static gboolean gui_text_scrolling_configure_event 	(GtkWidget *text_scrolling, GdkEventConfigure *event);
static gboolean gui_text_scrolling_button_press 	(GtkWidget *text_scrolling, GdkEventButton *event);
static gboolean gui_text_scrolling_button_release 	(GtkWidget *text_scrolling, GdkEventButton *event);
static gboolean gui_text_scrolling_motion_notify 	(GtkWidget *text_scrolling, GdkEventMotion *event);
static gboolean gui_text_scrolling_leave_notify_event (GtkWidget *text_scrolling, GdkEventCrossing *event);

typedef struct _TextRow TextRow;

struct _TextRow{
    gboolean running;
    gboolean done;
    gdouble ypos;
    gchar* text;
    TextRow *next;
};

typedef struct _GuiTextScrollingPrivate GuiTextScrollingPrivate;

struct _GuiTextScrollingPrivate
{
	guint timeout;
	gdouble ypos;
	gdouble active;
	TextRow* row;
};

static void
gui_text_scrolling_class_init (GuiTextScrollingClass *class)
{
	GObjectClass *obj_class;
	GtkWidgetClass *widget_class;

	obj_class = G_OBJECT_CLASS (class);
	obj_class->finalize = (GObjectFinalizeFunc) gui_text_scrolling_finalize;

	widget_class = GTK_WIDGET_CLASS (class);

	widget_class->expose_event 			= gui_text_scrolling_expose;
	widget_class->configure_event 		= gui_text_scrolling_configure_event;
	widget_class->button_press_event 	= gui_text_scrolling_button_press;
	widget_class->button_release_event 	= gui_text_scrolling_button_release;
	widget_class->motion_notify_event 	= gui_text_scrolling_motion_notify;
	widget_class->leave_notify_event 	= gui_text_scrolling_leave_notify_event;

	g_type_class_add_private (obj_class, sizeof (GuiTextScrollingPrivate));
}

static void
gui_text_scrolling_init (GuiTextScrolling *text_scrolling)
{
	GuiTextScrollingPrivate *priv;

	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (text_scrolling);
	priv->active = FALSE;

	gtk_widget_set_extension_events (GTK_WIDGET (text_scrolling), GDK_EXTENSION_EVENTS_CURSOR);

	gtk_widget_add_events (GTK_WIDGET (text_scrolling), GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_BUTTON_MOTION_MASK
			 | GDK_POINTER_MOTION_MASK);
}

static void
gui_text_scrolling_finalize (GuiTextScrolling *top_area)
{

}

static gboolean
gui_text_scrolling_timeout(gpointer data)
{   
    gboolean one_started = FALSE;
    gboolean done = TRUE;        
    GtkWidget* widget = GTK_WIDGET(data);
    
    TextRow* row = NULL;
    GuiTextScrolling *text_scrolling = GUI_TEXT_SCROLLING(data);
    GuiTextScrollingPrivate *priv = GUI_TEXT_SCROLLING_GET_PRIVATE(text_scrolling);      

    for(row = priv->row; row; row = row->next){
        if(row->running && !row->done){
            if(row->ypos < -20){
                row->done = TRUE;
            }else{
                row->ypos -= 1;     
                done = FALSE;
            }
        }else if(!row->running && !one_started && (priv->ypos > 20)){
            row->running = TRUE;
            row->ypos = widget->allocation.height / 2 + 50 + 30;            
            priv->ypos = 0;
            one_started = TRUE;
            done = FALSE;
        }else if(!row->running){
            done = FALSE;
        }
    }
    
    if(done){
        for(row = priv->row; row; row = row->next){
            row->running = FALSE;
            row->done = FALSE;
        }
    }
    
    gtk_widget_queue_draw(widget);
    
    priv->ypos += 1; 
    
    return priv->active;
}

static void
gui_text_scrolling_draw_text(GtkWidget* widget, cairo_t *cr, gchar* text, gdouble ypos)
{	
	gdouble xpos;
	cairo_text_extents_t te;
	
	g_return_if_fail(text);
	
	cairo_save(cr);		
	cairo_set_source_rgb (cr, 1, 1, 1);
	cairo_select_font_face(cr, "Georgia", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
	
    cairo_set_font_size(cr, 18);
    cairo_text_extents(cr, text, &te);    
    xpos = (widget->allocation.width - te.width)/2;    
		
	cairo_move_to(cr, xpos, ypos);
	cairo_show_text(cr, text);
	cairo_restore(cr);
}

static void
gui_text_scrolling_draw_bg(GtkWidget *widget, cairo_t *cr)
{
    cairo_pattern_t* mask = NULL;
    
    cairo_save (cr);
    cairo_set_source_rgb (cr, 0.368627451, 0.368627451, 0.368627451);
    mask = cairo_pattern_create_linear ( widget->allocation.width / 2, 60,
                                         widget->allocation.width / 2, 0);
	cairo_pattern_add_color_stop_rgba (mask, 0.0f,  0.368627451, 0.368627451, 0.368627451, 0.0f);
	cairo_pattern_add_color_stop_rgba (mask, 0.25f, 0.368627451, 0.368627451, 0.368627451, 0.01f);
	cairo_pattern_add_color_stop_rgba (mask, 0.5f,  0.368627451, 0.368627451, 0.368627451, 0.125f);
	cairo_pattern_add_color_stop_rgba (mask, 0.75f, 0.368627451, 0.368627451, 0.368627451, 0.5f);
	cairo_pattern_add_color_stop_rgba (mask, 1.0f,  0.368627451, 0.368627451, 0.368627451, 1.0f);
	cairo_mask (cr, mask);		
    cairo_pattern_destroy (mask);
    
    mask = cairo_pattern_create_linear ( widget->allocation.width / 2, widget->allocation.height - 60,
                                         widget->allocation.width / 2, widget->allocation.height);
	cairo_pattern_add_color_stop_rgba (mask, 0.0f,  0.368627451, 0.368627451, 0.368627451, 0.0f);
	cairo_pattern_add_color_stop_rgba (mask, 0.25f, 0.368627451, 0.368627451, 0.368627451, 0.01f);
	cairo_pattern_add_color_stop_rgba (mask, 0.5f,  0.368627451, 0.368627451, 0.368627451, 0.125f);
	cairo_pattern_add_color_stop_rgba (mask, 0.75f, 0.368627451, 0.368627451, 0.368627451, 0.5f);
	cairo_pattern_add_color_stop_rgba (mask, 1.0f,  0.368627451, 0.368627451, 0.368627451, 1.0f);
	cairo_mask (cr, mask);		
    cairo_pattern_destroy (mask);    
    
    cairo_restore (cr);
}

static void
gui_text_scrolling_draw(GtkWidget *widget, cairo_t *cr)
{	
    TextRow* row;
	GuiTextScrollingPrivate *priv;
    cairo_matrix_t   matrix;
	cairo_pattern_t* mask = NULL;

	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(widget));    
    
    cairo_save (cr);
    cairo_rectangle(cr, 0, 0, widget->allocation.width, 200);
    cairo_clip(cr);	        
    for(row = priv->row; row; row = row->next){
        if(row->running && !row->done) gui_text_scrolling_draw_text(widget, cr, row->text, row->ypos);    
    }    
    
    mask = cairo_pattern_create_linear ( 0.0f, 150, 0.0f, 200);
	cairo_pattern_add_color_stop_rgba (mask, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
	cairo_pattern_add_color_stop_rgba (mask, 0.25f, 0.0f, 0.0f, 0.0f, 0.01f);
	cairo_pattern_add_color_stop_rgba (mask, 0.5f, 0.0f, 0.0f, 0.0f, 0.125f);
	cairo_pattern_add_color_stop_rgba (mask, 0.75f, 0.0f, 0.0f, 0.0f, 0.5f);
	cairo_pattern_add_color_stop_rgba (mask, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f);
	cairo_mask (cr, mask);
    cairo_pattern_destroy (mask);
    cairo_restore (cr);
    
    cairo_save (cr);
	cairo_rectangle(cr, 0, 200, widget->allocation.width, widget->allocation.height);
    cairo_clip(cr);
	cairo_matrix_init_scale (&matrix, 1.0f, -1.0f);
	cairo_matrix_translate (&matrix,  0.0f, -250.0f);
	cairo_transform (cr, &matrix);
	        
    for(row = priv->row; row; row = row->next){        
        if(row->running && !row->done) gui_text_scrolling_draw_text(widget, cr, row->text, row->ypos - 150);    
    } 
    
    mask = cairo_pattern_create_linear ( 0.0f, 0.0f, 0.0f, 50);
	cairo_pattern_add_color_stop_rgba (mask, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
	cairo_pattern_add_color_stop_rgba (mask, 0.25f, 0.0f, 0.0f, 0.0f, 0.01f);
	cairo_pattern_add_color_stop_rgba (mask, 0.5f, 0.0f, 0.0f, 0.0f, 0.125f);
	cairo_pattern_add_color_stop_rgba (mask, 0.75f, 0.0f, 0.0f, 0.0f, 0.5f);
	cairo_pattern_add_color_stop_rgba (mask, 1.0f, 0.0f, 0.0f, 0.0f, 0.75);
	cairo_mask (cr, mask);	
    cairo_pattern_destroy (mask);
    cairo_restore (cr);
    
    gui_text_scrolling_draw_bg(widget, cr);
}

static gboolean
gui_text_scrolling_configure_event (GtkWidget *text_scrolling, GdkEventConfigure *event)
{
	return FALSE;
}

static gboolean
gui_text_scrolling_expose (GtkWidget *text_scrolling, GdkEventExpose *event)
{
    TextRow* row;
	cairo_t *cr;
	GuiTextScrollingPrivate *priv;

	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));
    
    cr = gdk_cairo_create(text_scrolling->window);	
    
    cairo_push_group (cr);        
    cairo_rectangle(cr, event->area.x, event->area.y, event->area.width, event->area.height);
    cairo_clip(cr);	    
    cairo_set_source_rgb(cr, 0, 0, 0);
    cairo_paint(cr);
    gui_text_scrolling_draw(text_scrolling, cr);
    
    cairo_pop_group_to_source (cr);
    cairo_paint(cr);    
    cairo_destroy(cr);

	return FALSE;
}

static gboolean
gui_text_scrolling_button_press (GtkWidget *text_scrolling, GdkEventButton *event)
{
	GuiTextScrollingPrivate *priv;

	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));

	return FALSE;
}

static gboolean
gui_text_scrolling_button_release (GtkWidget *text_scrolling, GdkEventButton *event)
{
	GuiTextScrollingPrivate *priv;

	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));

	return FALSE;
}

static gboolean
gui_text_scrolling_motion_notify (GtkWidget *text_scrolling, GdkEventMotion *event)
{
	GuiTextScrollingPrivate *priv;

	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));

	return FALSE;
}

static gboolean
gui_text_scrolling_leave_notify_event(GtkWidget *text_scrolling, GdkEventCrossing *event)
{
	GuiTextScrollingPrivate *priv;

	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));

	return FALSE;
}

static inline void
gui_text_scrolling_redraw(GtkWidget *text_scrolling)
{
	gui_text_scrolling_configure_event(text_scrolling, NULL);
    gtk_widget_queue_draw(text_scrolling);
}

void
gui_text_scrolling_set_text(GtkWidget *text_scrolling, const gchar* text)
{
    TextRow *tmp;
    TextRow *row;    
    GuiTextScrollingPrivate *priv;
	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));
	
	row = g_malloc0(sizeof(TextRow));
	row->text = g_strdup(text);;
	row->ypos = -1;
	row->running = FALSE;	

    for(tmp = priv->row; tmp; tmp = tmp->next){
        if (tmp->next == NULL){
            tmp->next = row;
            return;
        }
    }
    
    priv->row = row;
}

void
gui_text_scrolling_stop(GtkWidget* text_scrolling)
{
    GuiTextScrollingPrivate *priv;
	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));
    priv->active = FALSE;
}

void
gui_text_scrolling_start(GtkWidget* text_scrolling)
{
    GuiTextScrollingPrivate *priv;
	priv = GUI_TEXT_SCROLLING_GET_PRIVATE (GUI_TEXT_SCROLLING(text_scrolling));
	
    if(!priv->active){
        priv->active = TRUE;
        g_timeout_add(35, gui_text_scrolling_timeout, text_scrolling);
    }
}

GtkWidget *
gui_text_scrolling_new (void)
{
	return g_object_new (GUI_TYPE_TEXT_SCROLLING, NULL);
}

