/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "io_settings.h"
#include "io_hamachi_settings.h"

G_DEFINE_TYPE (IOHamachiSettings, io_hamachi_settings, G_TYPE_OBJECT)

#define IO_HAMACHI_SETTINGS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), IO_TYPE_HAMACHI_SETTINGS, IOHamachiSettingsPrivate))
#define HAMACHI_SETTINGS_FILE "/config"

typedef struct _IOHamachiSettingsPrivate IOHamachiSettingsPrivate;

struct _IOHamachiSettingsPrivate
{    
    gchar* settings_path;
    GSList* unknown_list;
};

IOHamachiSettings*
io_hamachi_settings_new(IOSettings *settings)
{      
    gchar* path;    
    IOHamachiSettings* hsettings;
    
    path = g_strconcat(settings->hamachi_folder, HAMACHI_SETTINGS_FILE, NULL);
    hsettings = IO_HAMACHI_SETTINGS(g_object_new (IO_TYPE_HAMACHI_SETTINGS, NULL)); 
    io_hamachi_settings_read_settings(hsettings, path);
    
    g_free(path);
    return hsettings;
}

static void
io_hamachi_settings_init(IOHamachiSettings* settings)
{    
    settings->keep_alive = -1;
    settings->udp_port = -1;
}

static void 
io_hamachi_settings_finalize(IOHamachiSettings* settings)
{
    io_hamachi_settings_write_settings(settings);
}

static void
io_hamachi_settings_class_init(IOHamachiSettingsClass* class)
{
    GObjectClass *obj_class;
    obj_class = G_OBJECT_CLASS (class);
    obj_class->finalize = (GObjectFinalizeFunc) io_hamachi_settings_finalize;
    
    g_type_class_add_private (obj_class, sizeof (IOHamachiSettingsPrivate));
}

gboolean
io_hamachi_settings_read_settings(IOHamachiSettings* settings, const gchar* path)
{    
    FILE *file;
    gchar* line;
    gchar** parts;
    IOHamachiSettingsPrivate *priv;
    
    priv = IO_HAMACHI_SETTINGS_GET_PRIVATE(settings);        
    if(path[0] == '~'){
        priv->settings_path = g_strconcat(g_get_home_dir(), ++path, NULL);
    }else{
        priv->settings_path = g_strdup(path);
    }
    
    file = fopen(priv->settings_path, "r");
    line = g_malloc(100 * sizeof(gchar));
    
    if(!file){ 
        g_printerr("Failed to open hamachi config file at %s\n", priv->settings_path);
        return;
    }
    
    while(fgets(line, 100, file) != NULL){
        parts = g_strsplit(line, " ", 10);
        
        if(parts[0] == NULL){
            g_strfreev(parts);
            continue;    
        }else if(g_ascii_strcasecmp(parts[0], "KeepAlive") == 0){
            if(parts[1] != NULL){
                settings->keep_alive = atoi(parts[1]);
                g_printerr("keep_alive: %d\n", settings->keep_alive);
            }
        }else if(g_ascii_strcasecmp(parts[0], "UdpPort") == 0){
            if(parts[1] != NULL){
                settings->udp_port = atoi(parts[1]);
                g_printerr("udp_port: %d\n", settings->udp_port);
            }
        }else{
            priv->unknown_list = g_slist_append(priv->unknown_list, g_strdup(line));
        }
        
        g_strfreev(parts);
    }
    
    fclose(file);
}

gboolean
io_hamachi_settings_write_settings(IOHamachiSettings* settings)
{
    FILE *file;
    GSList *list;
    IOHamachiSettingsPrivate *priv;
        
    priv = IO_HAMACHI_SETTINGS_GET_PRIVATE(settings); 
    file = fopen(priv->settings_path, "w");    
        
    if(!file){ 
        g_printerr("Failed to open hamachi config file at %s\n", priv->settings_path);
        return;
    }
    
    for(list = priv->unknown_list; list; list = g_slist_next(list)){
        fprintf(file, "%s\n", list->data);
    }
    
    if(settings->keep_alive != -1){
        fprintf(file, "KeepAlive %d\n", settings->keep_alive);
    }
    
    if(settings->udp_port != -1){
        fprintf(file, "UdpPort %d\n", settings->udp_port);
    }
    
    close(file);
}

