/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <sys/un.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <pthread.h> /** pthread_mutex_* */

#include <errno.h> /** EINVAL */

#include <stdio.h>
#include <string.h> /** memcpy */
#include <unistd.h>
#include <stdlib.h>
#include <glib.h>

#include "io_ipc.h"

#define IPC_STATUS_START_CODE -128

static gint	ipc_status_read(IPCConnection* connection, gchar* buffer, gint size);

const gchar*
ipc_error_string(IPCConnection *connection)
{
	switch(connection->error){
		case IPC_ERROR_SOCKET:
			return "IPC_ERROR_SOCKET";
		case IPC_ERROR_ARG:
			return "IPC_ERROR_ARG";
		case IPC_ERROR_PARSE:
			return "IPC_ERROR_PARSE";
		case IPC_ERROR_CONNECT:
			return "IPC_ERROR_CONNECT";
		case IPC_ERROR_OTHER:
			return "IPC_ERROR_OTHER";
		case IPC_ERROR_ALREADY_OFFLINE:
			return "IPC_ERROR_ALREADY_OFFLINE";
		case IPC_ERROR_ALREADY_ONLINE:
			return "IPC_ERROR_ALREADY_ONLINE";
		case IPC_ERROR_NO_ERROR:
			return "IPC_ERROR_NO_ERROR";
		case IPC_ERROR_NETWORK_EXIST:
			return "IPC_ERROR_NETWORK_EXIST";		
		case IPC_ERROR_NON_EXISTENT_NETWORK:
			return "IPC_ERROR_NON_EXISTENT_NETWORK";
		case IPC_ERROR_NOT_OWNER:
			return "IPC_ERROR_NOT_OWNER";
		default:
			return "UNKNOWN ERROR";
	}
}

IPCConnection* 
ipc_connect(gchar* path)
{
	gint sock;
	IPCConnection *connection;
	struct sockaddr_un addr;	

    g_return_if_fail(path);

	g_strlcpy((gchar*)&addr.sun_path, path, sizeof(addr.sun_path));
	addr.sun_family = AF_UNIX;

	if((sock = socket(PF_UNIX, SOCK_STREAM, 0)) == -1){
		perror("socket()");
		return NULL;
	}

	if((connect(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr_un))) == -1){
		perror("connect()");
		return NULL;
	}	

	connection = g_malloc0(sizeof(IPCConnection));
	connection->socket = sock;
	connection->error = IPC_ERROR_NO_ERROR;
	
	if(pthread_mutex_init (&connection->lock, NULL) != 0){
		g_printerr("Error creating mutex\n");
		return NULL;
	}
	
	return connection;
}

void
ipc_disconnect(IPCConnection* connection)
{
    g_return_if_fail(connection);
    
	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");
	
	close(connection->socket);	
	g_free(connection);
}

void
ipc_shutdown(IPCConnection* connection)
{
	gchar op;
	gint size;

    g_return_if_fail(connection);

	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");
	
	op = IPC_SHUTDOWN;
	size = htonl(1);
	
	if(send(connection->socket, &size, sizeof(gint), 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send()");
		return;
	}

	if(send(connection->socket, &op, sizeof(gchar), 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send()");
		return;
	}
	
	close(connection->socket);	
	g_free(connection);
}

gchar* 
ipc_get_sock_path(gchar* base)
{
	const gchar* home;
	gchar* sock;

    if(base == NULL){
        home = g_getenv("HOME");
    	sock = g_strconcat(home, HAMACHI_SOCK, NULL);
    	return sock;
    }else if(base[0] == '~'){
	    home = g_getenv("HOME");
    	sock = g_strconcat(home, HAMACHI_SOCK, NULL);
    	return sock;
    }else{
        sock = g_strconcat(base, "/ipc_sock", NULL);
        return sock;
    }	
}

IPCStatus*
ipc_status_get(IPCConnection* connection)
{
	gint size;
	gchar* buff;
	gchar* borg;
	IPCStatus* stat;	
	
	borg = buff = g_malloc0(1024);
	stat = g_malloc0(sizeof(IPCStatus));
	size = ipc_status_read(connection, buff, 1024);
	
	if(size == -1){ 
		connection->error = IPC_ERROR_SOCKET;
		return NULL; 		
	}
	
	if(buff[0] != IPC_STATUS_START_CODE){
		connection->error = IPC_ERROR_PARSE;
		return NULL;
	}
	
	stat->useragent = g_strndup(&buff[2], buff[1]);
	buff += buff[1] + 2;
	
	memcpy(&stat->pid, buff, sizeof(gint));
	stat->pid = ntohl(stat->pid);
	buff += sizeof(gint);
	
	buff += 5;
	stat->online = (gboolean)buff[0];
	
	buff += 1;
	stat->nick = g_strndup(&buff[1], buff[0]);
	
	g_free(borg);
	
	connection->error = IPC_ERROR_NO_ERROR;
	
	return stat;	
}

void
ipc_status_print(IPCStatus *status)
{
	g_printerr("useragent  : %s\n", status->useragent);
	g_printerr("pid        : %d\n", status->pid);
	g_printerr("logged in  : %s\n", (status->online)?"Yes":"No");
	g_printerr("nick       : %s\n", status->nick);
}

void
ipc_status_free(IPCStatus *status)
{
	g_return_if_fail(status);
	
	if(status->useragent)
		g_free(status->useragent);
	if(status->nick)
		g_free(status->nick);
	g_free(status);
}

gint 
ipc_status_read(IPCConnection* connection, gchar* buffer, gint buffer_size)
{
	gint err = 0;
	gint size = 0;
	gint left = 0;

	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");

	/** read size*/
	if((err = recv(connection->socket, &size, 4, 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("recv()");
		return 1;
	}

	left = size = ntohl(size);

	if(!buffer){
		buffer = g_malloc0(size);
		buffer_size = size;
	}

	if(size > buffer_size){
		connection->error = IPC_ERROR_ARG;
		return -1;
	}

	while(left > 0){
		if((err = recv(connection->socket, buffer, left, 0)) == -1){
			connection->error = IPC_ERROR_SOCKET;
			perror("recv status");
			return -1;
		}
		buffer += err;
		left -= err;		
	}

	connection->error = IPC_ERROR_NO_ERROR;
	
	if(pthread_mutex_unlock(&connection->lock) != 0)
		g_printerr("pthread_mutex_unlock error\n");
	
	return size;
}

void
ipc_set_nick(IPCConnection *connection, const gchar* nick)
{    
    gint t_len;
    gchar s_len;
    gchar buff[1024];
    
    s_len = (gchar)strlen(nick);
    t_len = s_len + 2;
    t_len = htonl(t_len);
    
    buff[0] = IPC_SET_NICK;
    buff[1] = s_len;
    
    memcpy(&buff[2], nick, s_len);
    
    if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");
	
	if(send(connection->socket, &t_len, sizeof(gint), 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(size)");
	}
		
	if(send(connection->socket, &buff, ntohl(t_len), 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(message)");
	}
		
    if(pthread_mutex_unlock(&connection->lock) != 0)
		g_printerr("pthread_mutex_unlock error\n");
		
    connection->error = IPC_ERROR_NO_ERROR;
}



