/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <stdio.h>
#include <unistd.h>
#include <glib.h>
#include <glib-object.h>
#include "io_settings.h"
#include "io_settings_marshallers.h"

G_DEFINE_TYPE (IOSettings, io_settings, G_TYPE_OBJECT)

#define IO_SETTINGS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), IO_TYPE_SETTINGS, IOSettingsPrivate))

static void 	io_settings_read_setting(IOSettings* settings);
static void		io_settings_open   		(IOSettings* settings);
static void 	io_settings_finalize   	(IOSettings* settings);
static void     io_settings_set_file    (IOSettings* settings, gchar* path);
static gchar* 	io_settings_get_path 	();

typedef struct _IOSettingsPrivate IOSettingsPrivate;

struct _IOSettingsPrivate
{
    GKeyFile* gkey;
    gchar* path;
};

enum{
    SIGNAL_SETTINGS_UPDATE,
    SIGNAL_LAST
};

static guint io_settings_signals[SIGNAL_LAST] = { 0 };

IOSettings*
io_settings_new(void)
{      
    gchar* path;
    IOSettings* settings;
    
    path = io_settings_get_path();
    settings = IO_SETTINGS(g_object_new (IO_TYPE_SETTINGS, NULL));    
    io_settings_set_file(settings, path);
    
    g_free(path);
    return settings;
}

IOSettings*
io_settings_new_with_path(gchar* path)
{
    IOSettings* settings;
    
    settings = IO_SETTINGS(g_object_new (IO_TYPE_SETTINGS, NULL));
    io_settings_set_file(settings, path);
    
    return settings;
}

void
io_settings_updated(IOSettings *settings)
{
    g_signal_emit(settings, io_settings_signals[SIGNAL_SETTINGS_UPDATE], 0, NULL);
}

static void
io_settings_init(IOSettings* settings)
{    
}

static void
io_settings_set_file(IOSettings* settings, gchar* path)
{
    IOSettingsPrivate *priv;
    
    priv = IO_SETTINGS_GET_PRIVATE(settings);
    priv->path = g_strdup(path);

    io_settings_open(settings);   
    io_settings_read_setting(settings);
}

static void
io_settings_class_init(IOSettingsClass* class)
{
    GObjectClass *obj_class;
    
    obj_class = G_OBJECT_CLASS (class);
    obj_class->finalize = (GObjectFinalizeFunc) io_settings_finalize;
    
    io_settings_signals[SIGNAL_SETTINGS_UPDATE] = g_signal_new (
			"settings-update",
			G_OBJECT_CLASS_TYPE (obj_class),
			G_SIGNAL_RUN_FIRST,
			0,NULL, NULL,
			_io_settings_marshal_VOID__VOID,
			G_TYPE_NONE, 0);
    
    g_type_class_add_private (obj_class, sizeof (IOSettingsPrivate));
}

static void 
io_settings_finalize(IOSettings* settings)
{
	io_settings_write_settings(settings);
}

static gchar*
io_settings_get_path()
{
    const gchar* home_dir;
    gchar* sett_dir;
    
    home_dir = g_get_home_dir();
    sett_dir = g_strconcat(home_dir, HAMACHI_SETTINGS_FILE, NULL);
    
    return sett_dir;
}

static void
io_settings_open(IOSettings* settings)
{
    GKeyFile* gkey;
    IOSettingsPrivate *priv;
    
    priv = IO_SETTINGS_GET_PRIVATE(settings);
    gkey = g_key_file_new();     
    
    if(!g_key_file_load_from_file(gkey, priv->path, 0, NULL)){
		g_printerr("Error loading file: %s\n", priv->path);
    }
    
    priv->gkey = gkey;
}

void
io_settings_write_settings(IOSettings* settings)
{
	gsize size;
    gchar* data;    
    FILE *file;
    IOSettingsPrivate *priv;    
    
    priv = IO_SETTINGS_GET_PRIVATE(settings);
    
	g_key_file_set_boolean(priv->gkey, "GUI", "ShowIP", settings->show_ip);           
	g_key_file_set_boolean(priv->gkey, "GUI", "ShowNick", settings->show_nick);    	
    g_key_file_set_boolean(priv->gkey, "GUI", "ShowTaskbar", settings->show_taskbar);
    g_key_file_set_boolean(priv->gkey, "GUI", "ShowOpacity", settings->show_opacity);        
    g_key_file_set_double(priv->gkey, "GUI", "Opacity", settings->opacity);        
    g_key_file_set_boolean(priv->gkey, "GUI", "ShowOfflineClient", settings->show_offline);
    g_key_file_set_boolean(priv->gkey, "GUI", "ShowQuitDialog", settings->show_quit_dialog);
    
    g_key_file_set_integer(priv->gkey, "GUI", "MainWindowColorR", settings->main_window_color.red);
    g_key_file_set_integer(priv->gkey, "GUI", "MainWindowColorG", settings->main_window_color.green);
    g_key_file_set_integer(priv->gkey, "GUI", "MainWindowColorB", settings->main_window_color.blue);
    g_key_file_set_string(priv->gkey, "GUI", "MainWindowFont", settings->fontstr);
    
    g_key_file_set_string(priv->gkey, "SYSTEM", "CmdVNC", settings->cmd_vnc);		
    g_key_file_set_string(priv->gkey, "SYSTEM", "CmdSudoSu", settings->cmd_sudo);
    g_key_file_set_string(priv->gkey, "SYSTEM", "CmdTunCfg", settings->cmd_tuncfg);
    g_key_file_set_string(priv->gkey, "SYSTEM", "CmdHamachi", settings->cmd_hamachi);
    g_key_file_set_string(priv->gkey, "SYSTEM", "CmdBrowser", settings->cmd_browser);
    g_key_file_set_string(priv->gkey, "SYSTEM", "CmdTerminal", settings->cmd_terminal);        
    g_key_file_set_string(priv->gkey, "SYSTEM", "FolderHamachi", settings->hamachi_folder);    
    
    data = g_key_file_to_data(priv->gkey, &size, NULL);
    file = fopen(priv->path, "w");
    size = fwrite(data, sizeof(gchar), size, file);    
    fclose(file);
    g_free(data);
}

static gchar*
io_settings_get_string(GKeyFile *key_file, gchar* group, gchar* key, gchar* default_value)
{
    GError* err = NULL;
    gchar* value;
    
    value = g_key_file_get_string(key_file, group, key, &err);
    
    if(err != NULL){
		if(err->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
	    	g_printerr("Cant find key %s\n", key);
		else if(err->code == G_KEY_FILE_ERROR_INVALID_VALUE)
	    	g_printerr("Cant parse key %s\n", key);
		value = g_strdup(default_value);
    }
    
    return value;
}

static gint
io_settings_get_int(GKeyFile *key_file, gchar* group, gchar* key, gint default_value)
{
    GError* err = NULL;
    gint value;
    
    value = g_key_file_get_double(key_file, group, key, &err);
    
    if(err != NULL){
		if(err->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
	    	g_printerr("Cant find key %s\n", key);
		else if(err->code == G_KEY_FILE_ERROR_INVALID_VALUE)
	    	g_printerr("Cant parse key %s\n", key);
		value = default_value;
    }
    
    return value;
}

static gdouble
io_settings_get_double(GKeyFile *key_file, gchar* group, gchar* key, gdouble default_value)
{
    GError* err = NULL;
    gdouble value;
    
    value = g_key_file_get_double(key_file, group, key, &err);
    
    if(err != NULL){
		if(err->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
	    	g_printerr("Cant find key %s\n", key);
		else if(err->code == G_KEY_FILE_ERROR_INVALID_VALUE)
	    	g_printerr("Cant parse key %s\n", key);
		value = default_value;
    }
    
    return value;
}

static gboolean
io_settings_get_boolean(GKeyFile *key_file, gchar* group, gchar* key, gboolean default_value)
{
    GError* err = NULL;
    gboolean value;
    
    value = g_key_file_get_boolean(key_file, group, key, &err);
    
    if(err != NULL){
		if(err->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
	    	g_printerr("Cant find key %s\n", key);
		else if(err->code == G_KEY_FILE_ERROR_INVALID_VALUE)
	    	g_printerr("Cant parse key %s\n", key);
		value = default_value;
    }
    
    return value;
}

static void 
io_settings_read_setting(IOSettings* settings)
{    
    IOSettingsPrivate *priv;
    
    priv = IO_SETTINGS_GET_PRIVATE(settings);
    
    settings->show_ip = io_settings_get_boolean(priv->gkey, "GUI", "ShowIP", TRUE);
    settings->show_nick = io_settings_get_boolean(priv->gkey, "GUI", "ShowNick", TRUE);
    settings->show_taskbar = io_settings_get_boolean(priv->gkey, "GUI", "ShowTaskbar", TRUE);
    settings->show_opacity = io_settings_get_boolean(priv->gkey, "GUI", "ShowOpacity", FALSE);
    settings->opacity = io_settings_get_double(priv->gkey, "GUI", "Opacity", 0.7);    	
    settings->show_offline = io_settings_get_boolean(priv->gkey, "GUI", "ShowOfflineClient", TRUE);
    settings->show_quit_dialog = io_settings_get_boolean(priv->gkey, "GUI", "ShowQuitDialog", TRUE);
    
    settings->main_window_color.red = io_settings_get_int(priv->gkey, "GUI", "MainWindowColorR", 113*(65535/255));
    settings->main_window_color.green = io_settings_get_int(priv->gkey, "GUI", "MainWindowColorG", 113*(65535/255));
    settings->main_window_color.blue = io_settings_get_int(priv->gkey, "GUI", "MainWindowColorB", 113*(65535/255));
    settings->fontstr = io_settings_get_string(priv->gkey, "GUI", "MainWindowFont", "Sans 10S");
    
    settings->cmd_vnc = io_settings_get_string(priv->gkey, "SYSTEM", "CmdVNC", "vncviewer");
    settings->cmd_sudo = io_settings_get_string(priv->gkey, "SYSTEM", "CmdSudoSu", "gksudo");
    settings->cmd_tuncfg = io_settings_get_string(priv->gkey, "SYSTEM", "CmdTunCfg", "tuncfg");
    settings->cmd_hamachi = io_settings_get_string(priv->gkey, "SYSTEM", "CmdHamachi", "hamachi");
    settings->cmd_browser = io_settings_get_string(priv->gkey, "SYSTEM", "CmdBrowser", "nautilus");
    settings->cmd_terminal = io_settings_get_string(priv->gkey, "SYSTEM", "CmdTerminal", "gnome-terminal");        
    settings->hamachi_folder = io_settings_get_string(priv->gkey, "SYSTEM", "FolderHamachi", "~/.hamachi");
}
