/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <glib/gi18n.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h> 	/** waitpid()*/
#include <sys/wait.h> 	/** waitpid()*/

#include "gui_top_area.h"
#include "gui_bottom_area.h"
#include "gui_middle_area.h"
#include "io_misc.h"
#include "io_update.h"
#include "io_ipc.h"
#include "main.h"

#include <libintl.h>
#include <locale.h>
#include "config.h"



enum errors{
  	HAMACHI_NOT_STARTED,
  	TUNCFG_NOT_STARTED,
  	NO_ERROR
};

static gint
fix_errors(MainPrivate* priv, gint err)
{
	FILE* out;
	gchar msg[512];
	gchar* cmd;
	gint count;

	switch(err){
	case HAMACHI_NOT_STARTED:
	    cmd = g_strconcat(priv->settings->cmd_hamachi, " start", NULL);
		out = popen(cmd, "r");
		count = fread(&msg, 1, 512, out);
		pclose(out);
		g_free(cmd);

		if(count > 0){
			if(g_strrstr((gchar*)&msg, "tap: connect()") != NULL)
				return TUNCFG_NOT_STARTED;
		}else{			
			cmd = g_strconcat(priv->settings->cmd_hamachi, " login", NULL);
			out = popen(cmd, "r");
			count = fread(&msg, 1, 512, out);
			pclose(out);
			g_free(cmd);
			return NO_ERROR;
		}
	case TUNCFG_NOT_STARTED:
	    cmd = g_strconcat(priv->settings->cmd_sudo, " ", priv->settings->cmd_tuncfg, NULL);
		out = popen(cmd, "r");
		pclose(out);
		g_free(cmd);
		return HAMACHI_NOT_STARTED;

	default:
		break;
	}

	return NO_ERROR;
}

static void
io_update_inner_loop(MainPrivate* priv)
{
	IPCNetwork *old_net = NULL;	
	IPCNetwork *new_net = NULL;
	    
	for(;;){
		if(priv->connection == NULL){
			io_update_loop(priv);
			return;
		}		
		
		new_net = ipc_list_get(priv->connection);
		
		if(priv->connection->error != IPC_ERROR_NO_ERROR){
			g_printerr("IPC error: %s\n", ipc_error_string(priv->connection));
			ipc_disconnect(priv->connection);
			priv->connection = NULL;
			io_update_loop(priv);
			return;
		}
		
		if(!ipc_list_equal(new_net, old_net)){					
			if(!ipc_list_nicks_check(new_net)){
				gdk_threads_enter();	
				gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), _("Retrieving nicks..."));
				gdk_threads_leave();
				ipc_list_nicks_get(priv->connection);
				ipc_list_free(new_net);
			}else{							
				gdk_threads_enter();
				gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), priv->stats->nick);	
				gui_middle_area_set_network(GUI_MIDDLE_AREA(priv->middle_area), new_net);		
				gdk_threads_leave();
				ipc_list_free(old_net);
				old_net = new_net;
			}
		}else{
		    ipc_list_free(new_net);
        }
		waitpid(-1, NULL, WNOHANG);
		g_usleep(G_USEC_PER_SEC * 2);	
	}    
}

void
io_update_loop (MainPrivate* priv)
{
	gint err = HAMACHI_NOT_STARTED;
	gchar* path = NULL;	
	gchar* addr;

	for(;;){
        while(priv->hamchi_user_stop)
            g_usleep(G_USEC_PER_SEC * 1);
        
        path = ipc_get_sock_path(priv->settings->hamachi_folder);	
        addr = io_misc_get_addr();	
        priv->connection = ipc_connect(path);
        
        if(priv->connection == NULL){
            while(err != NO_ERROR){
                if(err == HAMACHI_NOT_STARTED){
                    gdk_threads_enter();
                    gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), _("Starting Hamachi..."));
                    gdk_threads_leave();
                }else if(err == TUNCFG_NOT_STARTED){
                    gdk_threads_enter();
                    gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), _("Starting Tuncfg..."));
                    gdk_threads_leave();			
                }
                
                err = fix_errors(priv, err);                
                g_usleep(G_USEC_PER_SEC * 1);
            }
            g_usleep(G_USEC_PER_SEC * 1);
            continue;
        }		
        
        priv->stats = ipc_status_get(priv->connection);
        
        if(priv->stats == NULL){
            g_printerr("Failed to read stats, make sure that hamachi-gui is not already running!\n");
            gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), _("Internal Error..."));
            ipc_disconnect(priv->connection);
            priv->connection = NULL;
            g_usleep(G_USEC_PER_SEC * 5);
            continue;
        }
        
        gdk_threads_enter();
        gui_top_area_set_addr(GUI_TOP_AREA(priv->top_area), addr);
        gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), priv->stats->nick);
        gdk_threads_leave();
        g_free(path);

        io_update_inner_loop(priv);
    }
}
